#!/usr/bin/python3
import os
import sys
import argparse
import re
from math import floor
from PIL import Image



def get_dot_index(image_file):
    return re.search(r'\.(?!.*\.)(?=.+)', image_file).start()




def split(targets, splits, axis):
    """DOCSTRING TODO"""
    for target in targets:
        with Image.open(target) as image:

            if axis == 'h':
                maxis = image.width

            elif axis == 'v':
                maxis = image.height

            sizes = [floor(maxis/splits) for _ in range(splits)]

            if maxis%splits:
                for x in range(maxis%splits):
                    sizes[-x-1] += 1

            coords = [sum(sizes[:x]) for x in range(len(sizes))]

            for size, coord, count in [[sizes[x], coords[x], x] for x in range(len(sizes))]:
                dot_index = get_dot_index(target)
                output = target[:dot_index] + f'_{count}' + target[dot_index:]

                if axis == 'h':
                    os.system(f"magick -extract {size}x{image.height}+{coord}+0 '{target}' '{output}'")

                elif axis == 'v':
                    os.system(f"magick -extract {image.width}x{size}+0+{coord} '{target}' '{output}'")




def merge(targets, axis):
    """DOCSTRING TODO"""

    if axis == 'h':
        sign = '+'

    elif axis == 'v':
        sign = '-'

    targets_string = ""
    for t in targets:
        targets_string += f"'{t}'" + ' '

    dot_index = get_dot_index(targets[0])
    output = targets[0][:dot_index] + '_merged' + targets[0][dot_index:]

    os.system(f"magick {targets_string}{sign}append '{output}'")




def main():
    """DOCSTRING TODO"""
    os.chdir(os.getcwd())
    parser = argparse.ArgumentParser(description='Split and remerge images.')

    parser.add_argument('targets',
                        nargs='+',
                        type=str,
                        help='Images or folders to process. Non-recursive. Folders must contain only images.')

    parser.add_argument('splits',
                        type=int,
                        help='Amount of sub-images to be split into.')

    parser.add_argument('-c',
                        action='store_const',
                        const=True,
                        dest='merge',
                        help='Reverse the process by combining the images in order of given. Splits value ignored.')

    parser.add_argument('-m',
                        choices=['h', 'v', 's'],
                        default='h',
                        dest='mode',
                        help='Mode. Horizontal (h), vertical (v), or subdivide (s). Subdivision TODO')

    args = parser.parse_args(sys.argv[1:])

    for t in args.targets.copy():
        i = args.targets.index(t)

        if os.path.isdir(t):
            folder = os.path.realpath(args.targets.pop(i))
            files = os.listdir(folder)
            files.sort()
            args.targets += [os.path.join(folder, x) for x in files]

        else:
            args.targets[i] = os.path.realpath(t)


    if args.mode == 's':
        print('TODO')

    elif args.merge:
        merge(args.targets, args.mode)

    else:
        split(args.targets, args.splits, args.mode)




if __name__ == "__main__":
    main()
